/*
    This program demonstrates how a "race condition" can come up
    when two threads both manipulate a shared variable. In this
    example, the shared variable is the global variable count.
    A race condition is where the result of a piece of code
    depends on the order in which the processor executes the
    threads.

    If you run this program on a multicore machine, you will see
    a fair number (but not a lot) of race conditions. However, if
    you run this program on a single core machine (or restrict the
    affinity of this program, and all of its threads, to a single
    core), then you will see very few race conditions. This shows
    how rare a race condition can be and why race conditions can be
    so hard to detect in real programs.
*/
#include <windows.h>
#include <stdio.h>

// global variables
int runFlag = TRUE;
unsigned int count  = 0;  // this variable is shared by thread1 and thread2
unsigned int count1 = 0;  // this variable is used only by thread1
unsigned int count2 = 0;  // this variable is used only by thread2

int main( int argc, char *argv[])
{
   DWORD WINAPI thread1(LPVOID);  //thread function prototypes
   DWORD WINAPI thread2(LPVOID);

   int threadParam1 = 0;  // thread function parameter
   int threadParam2 = 0;

   //Sleep(20000);  // use this time to set the processor affinity using Task Manager

   // create threads
   printf("race1: Starting threads\n");
   CreateThread(NULL, 0, thread1, &threadParam1, 0, NULL);
   CreateThread(NULL, 0, thread2, &threadParam2, 0, NULL);

   // let the threads run for a while
   Sleep(2000);
   // now stop the threads
   runFlag = FALSE;
   Sleep(20);

   printf("\nThreads stopped.\n");
   printf("count = %d, count1 = %d, count2 = %d\n", count, count1, count2);

   return 0;
}//main


DWORD WINAPI thread1(LPVOID threadParam)
{
   unsigned int new_count, old_count;
   while(runFlag)
   {
      // create a race situation with thread2
      // increment a global variable and store the results locally
      old_count = count++;
      new_count = count;
      // check that the local and global variables were consistent with each other
      if (new_count - old_count != 1)
         printf("Bingo1: new_count - old_count = %d\n", new_count-old_count);
   }
   return 0;
}//thread1


DWORD WINAPI thread2(LPVOID threadParam)
{
   unsigned int new_count, old_count;
   while(runFlag)
   {
      // create a race situation with thread1
      old_count = count++;
      new_count = count;
      // check for the race condition
      if (new_count - old_count != 1)
         printf("Bingo2: new_count - old_count = %d\n", new_count-old_count);
   }
   return 0;
}//thread2
